@@ -1,10 +1,11 @@
 pub mod attr;
+pub(crate) mod build;
 mod ctxt;
+mod expansion;
 mod mode;
-mod needs;
 pub(crate) mod symbol;
 pub(crate) mod tokens;
 
 pub(crate) use self::ctxt::Ctxt;
+pub(crate) use self::expansion::Expansion;
 pub(crate) use self::mode::{Mode, ModePath};
-pub(crate) use self::needs::{Needs, NeedsKind};
