@@ -1,30 +1,27 @@
-use proc_macro2::{Span, TokenStream};
-use quote::quote;
+use proc_macro2::Span;
 
 pub(crate) struct Tokens {
     pub(crate) decode_t: syn::ExprPath,
     pub(crate) decoder_t: syn::ExprPath,
-    pub(crate) decoder_var: syn::Ident,
-    pub(crate) default_t: TokenStream,
+    pub(crate) default_function: syn::ExprPath,
+    pub(crate) default_mode: syn::ExprPath,
     pub(crate) encode_t: syn::ExprPath,
     pub(crate) encoder_t: syn::ExprPath,
-    pub(crate) encoder_var: syn::Ident,
     pub(crate) error_t: syn::ExprPath,
     pub(crate) fmt: syn::ExprPath,
+    pub(crate) mode_t: syn::ExprPath,
     pub(crate) pack_decoder_t: syn::ExprPath,
-    pub(crate) pair_decoder_t: syn::ExprPath,
     pub(crate) pair_decoder_t_first: syn::ExprPath,
-    pub(crate) variant_decoder_t: syn::ExprPath,
-    pub(crate) variant_decoder_t_tag: syn::ExprPath,
-    pub(crate) variant_encoder_t: syn::ExprPath,
+    pub(crate) pair_decoder_t: syn::ExprPath,
     pub(crate) pair_encoder_t: syn::ExprPath,
     pub(crate) pairs_decoder_t: syn::ExprPath,
     pub(crate) pairs_encoder_t: syn::ExprPath,
-    pub(crate) phantom_data: syn::ExprPath,
-    pub(crate) value_visitor_t: syn::ExprPath,
     pub(crate) sequence_encoder_t: syn::ExprPath,
-    pub(crate) default_mode: syn::ExprPath,
-    pub(crate) mode_t: syn::ExprPath,
+    pub(crate) variant_decoder_t_tag: syn::ExprPath,
+    pub(crate) variant_decoder_t: syn::ExprPath,
+    pub(crate) variant_encoder_t: syn::ExprPath,
+    pub(crate) visit_string_fn: syn::ExprPath,
+    pub(crate) as_decoder_t: syn::ExprPath,
 }
 
 impl Tokens {
@@ -32,27 +29,25 @@ impl Tokens {
         Self {
             decode_t: path(span, prefix, ["de", "Decode"]),
             decoder_t: path(span, prefix, ["de", "Decoder"]),
-            decoder_var: syn::Ident::new("decoder", span),
-            default_t: quote!(::core::default::Default::default()),
+            default_function: core_path(span, ["default", "Default", "default"]),
+            default_mode: path(span, prefix, ["mode", "DefaultMode"]),
             encode_t: path(span, prefix, ["en", "Encode"]),
             encoder_t: path(span, prefix, ["en", "Encoder"]),
-            encoder_var: syn::Ident::new("encoder", span),
             error_t: path(span, prefix, ["error", "Error"]),
             fmt: core_path(span, ["fmt"]),
+            mode_t: path(span, prefix, ["mode", "Mode"]),
             pack_decoder_t: path(span, prefix, ["de", "PackDecoder"]),
-            pair_decoder_t: path(span, prefix, ["de", "PairDecoder"]),
             pair_decoder_t_first: path(span, prefix, ["de", "PairDecoder", "first"]),
-            variant_decoder_t: path(span, prefix, ["de", "VariantDecoder"]),
-            variant_decoder_t_tag: path(span, prefix, ["de", "VariantDecoder", "tag"]),
-            variant_encoder_t: path(span, prefix, ["en", "VariantEncoder"]),
+            pair_decoder_t: path(span, prefix, ["de", "PairDecoder"]),
             pair_encoder_t: path(span, prefix, ["en", "PairEncoder"]),
             pairs_decoder_t: path(span, prefix, ["de", "PairsDecoder"]),
             pairs_encoder_t: path(span, prefix, ["en", "PairsEncoder"]),
-            phantom_data: core_path(span, ["marker", "PhantomData"]),
-            value_visitor_t: path(span, prefix, ["de", "ValueVisitor"]),
             sequence_encoder_t: path(span, prefix, ["en", "SequenceEncoder"]),
-            default_mode: path(span, prefix, ["mode", "DefaultMode"]),
-            mode_t: path(span, prefix, ["mode", "Mode"]),
+            variant_decoder_t_tag: path(span, prefix, ["de", "VariantDecoder", "tag"]),
+            variant_decoder_t: path(span, prefix, ["de", "VariantDecoder"]),
+            variant_encoder_t: path(span, prefix, ["en", "VariantEncoder"]),
+            visit_string_fn: path(span, prefix, ["utils", "visit_string_fn"]),
+            as_decoder_t: path(span, prefix, ["de", "AsDecoder"]),
         }
     }
 }
